package italo.comecome.conf;

import italo.comecome.audio.Audio;
import italo.comecome.gui.figura.Figura;
import italo.comecome.logica.Fase;
import italo.comecome.logica.Jogo;
import italo.comecome.logica.Tabuleiro;
import italo.comecome.logica.TabuleiroException;
import italo.comecome.personagem.ComeCome;
import italo.comecome.personagem.Monstrinho;
import italo.comecome.personagem.Personagem;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.AccessControlException;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.swing.JOptionPane;

public class Conf {
	
	public static String CAMINHO_IMAGENS_TABULEIRO = "img/tabuleiro/1/";
	public static String EXTENCAO_IMAGENS_TABULEIRO = ".gif";
	
	public static String CAMINHO_JOGO = "conf/JOGO.PROPERTIES";
	public static String CAMINHO_IMAGENS = "conf/IMAGENS.BIN";
	public static String CAMINHO_SONS = "conf/SONS.BIN";
	public static String CAMINHO_FASES = "conf/fases/";	
	public static String NOME_ARQUIVO_FASE = "/FASE.PROPERTIES";
	public static String NOME_ARQUIVO_HORIZONTAL_LIVRE = "/HORIZONTAL_LIVRE.BIN";
	public static String NOME_ARQUIVO_VERTICAL_LIVRE = "/VERTICAL_LIVRE.BIN";
	public static String NOME_ARQUIVO_BOLINHAS = "/BOLINHAS.BIN";
	public static String NOME_ARQUIVO_BOLINHAS_GRANDES = "/BOLINHAS_GRANDES.BIN";
	public static String NOME_ARQUIVO_INDICES_PORTAS = "/INDICES_PORTAS.BIN";
	public static String NOME_ARQUIVO_DIRECOES_INICIAIS_MONSTRINHOS = "/DIRECOES_INICIAIS_MONSTRINHOS.BIN";
	public static String NOME_ARQUIVO_POSICOES_CELULAS_INICIAIS_COMECOME = "/POSICOES_CELULAS_INICIAIS_COMECOME.BIN";
	public static String NOME_ARQUIVO_POSICOES_CELULAS_INICIAIS_MONSTRINHOS = "/POSICOES_CELULAS_INICIAIS_MONSTRINHOS.BIN";
	
	public static String PROPRIEDADE_NUMERO_DE_VIDAS = "numerodevidas";
	public static String PROPRIEDADE_DIFICULDADE = "dificuldade";
	public static String PROPRIEDADE_NUMERO_DE_FASES = "numerodefases";
	public static String PROPRIEDADE_NUMERO_DE_BOLINHAS = "numerodebolinhas";
	public static String PROPRIEDADE_NUMERO_DE_BOLINHAS_GRANDES = "numerodebolinhasgrandes";
	public static String PROPRIEDADE_SLEEP_COMECOME = "sleepcomecome";
	public static String PROPRIEDADE_SLEEP_MONSTRINHOS = "sleepmonstrinhos";
	public static String PROPRIEDADE_INDICES_MONSTRINHOS = "indicesmonstrinhos";
	public static String PROPRIEDADE_NUMERO_DE_MONSTRINHOS = "numerodemonstrinhos";
	public static String PROPRIEDADE_NUMERO_DE_CELULAS_NA_HORIZONTAL = "numerocelulashorizontal";
	public static String PROPRIEDADE_NUMERO_DE_CELULAS_NA_VERTICAL = "numerocelulasvertical";
	public static String PROPRIEDADE_FATOR_DIMENSAO_INICIAL = "fatordimensaoinicial";
	public static String PROPRIEDADE_FATORES_MINI_CELULA = "fatoresminicelula";
		
	private static Conf conf;
	
	static {
		conf = new Conf();
	}
	
	public static Conf getInstance() {
		return conf;
	}
	
	public void configurarJogo(Jogo jogo, Figura figura, Audio audio, int larguraCompleta, int alturaCompleta) {
		try {
			File fileJogo = new File(CAMINHO_JOGO);
			Properties pJogo = new Properties();
			pJogo.load(new FileInputStream(fileJogo.getAbsoluteFile())); 
			String nVidas = pJogo.getProperty(PROPRIEDADE_NUMERO_DE_VIDAS);
			String dificuldade = pJogo.getProperty(PROPRIEDADE_DIFICULDADE);
			String nFases = pJogo.getProperty(PROPRIEDADE_NUMERO_DE_FASES);
			String caminhoImagens = ConfUtil.getConteudoArquivo(CAMINHO_IMAGENS);
			String caminhoSons = ConfUtil.getConteudoArquivo(CAMINHO_SONS); 
			carregarImagens(figura, caminhoImagens);
			carregarSons(audio, caminhoSons);
			jogo.setNVidas(Integer.parseInt(nVidas));
			jogo.setDificuldade(Integer.parseInt(dificuldade));
			Fase[] fases = getFases(jogo, larguraCompleta, alturaCompleta, Integer.parseInt(nFases));
			jogo.setFases(fases);			
		} catch(FileNotFoundException e) {
			Conf2.getInstance().configurarJogo(jogo, figura, audio, larguraCompleta, alturaCompleta);
		} catch(AccessControlException e) {
			Conf2.getInstance().configurarJogo(jogo, figura, audio, larguraCompleta, alturaCompleta);
		} catch(Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, e.getStackTrace(), "Falha na Configurao", JOptionPane.ERROR_MESSAGE);
			System.exit(1);			
		}
	}
			
	private void carregarImagens(Figura figura, String caminhosImagens) {
		StringTokenizer tokenizer = new StringTokenizer(caminhosImagens);
		String[] caminhos = new String[tokenizer.countTokens()];
		for(int i = 0; tokenizer.hasMoreTokens(); i++)
			caminhos[i] = tokenizer.nextToken();
		figura.inicializar(caminhos);
	}
	
	private void carregarSons(Audio audio, String caminhosSons) {
		StringTokenizer tokenizer = new StringTokenizer(caminhosSons);
		String[] caminhos = new String[tokenizer.countTokens()];
		for(int i = 0; tokenizer.hasMoreTokens(); i++)
			caminhos[i] = tokenizer.nextToken();
		audio.inicializar(caminhos);
	}
		
	private Fase[] getFases(Jogo jogo, int largura, int altura, int nFases) throws IOException {		
		Fase[] fases = new Fase[nFases];
		for(int i = 0; i < fases.length; i++) {			
			fases[i] = new Fase();
			carregarFase(jogo, largura, altura, fases[i], (i+1));			
		}
		return fases;
	}
	
	private void carregarFase(Jogo jogo, int largura, int altura, Fase fase, int numero) throws IOException {
		Properties pFase = new Properties();
		File file = new File(CAMINHO_FASES+numero+NOME_ARQUIVO_FASE);
		pFase.load(new FileInputStream(file.getAbsoluteFile()));
		int nBolinhas = Integer.parseInt(pFase.getProperty(PROPRIEDADE_NUMERO_DE_BOLINHAS));
		int nBolinhasGrandes = Integer.parseInt(pFase.getProperty(PROPRIEDADE_NUMERO_DE_BOLINHAS_GRANDES));;		
		int sleepComeCome = Integer.parseInt(pFase.getProperty(PROPRIEDADE_SLEEP_COMECOME));
		int nMonstrinhos = Integer.parseInt(pFase.getProperty(PROPRIEDADE_NUMERO_DE_MONSTRINHOS));
		int nCelulasHorizontal = Integer.parseInt(pFase.getProperty(PROPRIEDADE_NUMERO_DE_CELULAS_NA_HORIZONTAL));
		int nCelulasVertical = Integer.parseInt(pFase.getProperty(PROPRIEDADE_NUMERO_DE_CELULAS_NA_VERTICAL));
		int fatorDimensaoInicial = Integer.parseInt(pFase.getProperty(PROPRIEDADE_FATOR_DIMENSAO_INICIAL)); 
		String sleepMonstrinhos = pFase.getProperty(PROPRIEDADE_SLEEP_MONSTRINHOS);
		String indicesMonstrinhos = pFase.getProperty(PROPRIEDADE_INDICES_MONSTRINHOS);
		String fatoresMiniCelula = pFase.getProperty(PROPRIEDADE_FATORES_MINI_CELULA);
		String horizontalLivre = ConfUtil.getConteudoArquivo(CAMINHO_FASES+numero+NOME_ARQUIVO_HORIZONTAL_LIVRE); 
		String verticalLivre = ConfUtil.getConteudoArquivo(CAMINHO_FASES+numero+NOME_ARQUIVO_VERTICAL_LIVRE); 
		String bolinhas = ConfUtil.getConteudoArquivo(CAMINHO_FASES+numero+NOME_ARQUIVO_BOLINHAS); 
		String bolinhasGrandes = ConfUtil.getConteudoArquivo(CAMINHO_FASES+numero+NOME_ARQUIVO_BOLINHAS_GRANDES); 
		String direcoesIniciaisMonstrinhos = ConfUtil.getConteudoArquivo(CAMINHO_FASES+numero+NOME_ARQUIVO_DIRECOES_INICIAIS_MONSTRINHOS); 
		String indicesPortas = ConfUtil.getConteudoArquivo(CAMINHO_FASES+numero+NOME_ARQUIVO_INDICES_PORTAS); 
		String posicoesCelulaIniciaisComeCome = ConfUtil.getConteudoArquivo(CAMINHO_FASES+numero+NOME_ARQUIVO_POSICOES_CELULAS_INICIAIS_COMECOME); 
		String posicoesCelulasIniciaisMonstrinhos = ConfUtil.getConteudoArquivo(CAMINHO_FASES+numero+NOME_ARQUIVO_POSICOES_CELULAS_INICIAIS_MONSTRINHOS);
		String caminhoImagemTabuleiro = CAMINHO_IMAGENS_TABULEIRO+numero+EXTENCAO_IMAGENS_TABULEIRO;
		Tabuleiro tabuleiro = getTabuleiro(largura, altura, fatorDimensaoInicial, nCelulasHorizontal, nCelulasVertical, fatoresMiniCelula);
		
		fase.setTabuleiro(tabuleiro);
		fase.setNumero(numero); 
		fase.setComeCome(getComeCome(jogo, fase, sleepComeCome));
		fase.setMonstrinhos(getMonstrinhosFase(jogo, fase, sleepMonstrinhos, indicesMonstrinhos, nMonstrinhos));		
		fase.setCaminhoImagemTabuleiro(caminhoImagemTabuleiro);		
		fase.setNBolinhasIniciais(nBolinhas);
		fase.setNBolinhasGrandesIniciais(nBolinhasGrandes);			
		fase.setHorizontalLivre(ConfUtil.getVetorBoolean(horizontalLivre));
		fase.setVerticalLivre(ConfUtil.getVetorBoolean(verticalLivre));
		fase.setBolinhasIniciais(ConfUtil.getVetorBoolean(bolinhas));
		fase.setBolinhasGrandesIniciais(ConfUtil.getVetorInteger(bolinhasGrandes));
		fase.setDirecoesIniciaisMonstrinhos(ConfUtil.getVetorDirecoesIniciaisMonstrinhos(direcoesIniciaisMonstrinhos, nMonstrinhos, tabuleiro.getFatorMiniCelula()));
		fase.setIndicesPortas(ConfUtil.getVetorInteger(indicesPortas));
		fase.setPosicaoCelulaInicialComecome(ConfUtil.getVetorInteger(posicoesCelulaIniciaisComeCome));
		fase.setPosicoesCelulasIniciaisMonstrinhos(ConfUtil.getVetorInteger(posicoesCelulasIniciaisMonstrinhos));
	}
	
	private Tabuleiro getTabuleiro(int larguraCompleta, int alturaCompleta, 
				int fatorDimTabuleiro, int nCelulasHorizontal, 
				int nCelulasVertical, String fatoresMiniCelula) {
		Tabuleiro tabuleiro = null;		
		StringTokenizer tokenizer = new StringTokenizer(fatoresMiniCelula);
		int[] fatoresMC = new int[tokenizer.countTokens()];
		for(int i = 0; tokenizer.hasMoreTokens(); i++)
			fatoresMC[i] = Integer.parseInt(tokenizer.nextToken());
		try {
			tabuleiro = new Tabuleiro(larguraCompleta, alturaCompleta, 
				fatorDimTabuleiro, nCelulasHorizontal, nCelulasVertical, fatoresMC);
		} catch(TabuleiroException e) {
			JOptionPane.showMessageDialog(null, e);
		}
		return tabuleiro;
	}
	
	private Personagem[] getMonstrinhosFase(Jogo jogo, Fase fase, String sleepMonstrinhos, String indicesMonstrinhos, int nMonstrinhos) {
		Personagem[] monstrinhos = new Personagem[nMonstrinhos];
		StringTokenizer tokenizer = new StringTokenizer(sleepMonstrinhos);
		StringTokenizer tokenizer2 = new StringTokenizer(indicesMonstrinhos);	
		for(int i = 0; i < nMonstrinhos; i++) {
			int sleepM = Integer.parseInt(tokenizer.nextToken());
			int indice = Integer.parseInt(tokenizer2.nextToken());
			monstrinhos[i] = new Monstrinho(jogo, "monstrinhos", indice, sleepM);
		}
		return monstrinhos;
	}
	
	private ComeCome getComeCome(Jogo jogo, Fase fase, int sleepComeCome) {
		return new ComeCome(jogo, "comecome", 0, sleepComeCome);
	}
					
}
