package italo.comecome.entrada;

import italo.comecome.gui.ContainerJogo;

import java.awt.event.KeyListener;
import java.io.IOException;

import javax.swing.JOptionPane;

import com.centralnexus.input.Joystick;
import com.centralnexus.input.JoystickListener;

public class Entrada {

	public final static int MAX_NUMERO_JOYSTICKS = 2;
	
	private Teclado teclado;	
	private ContainerJogo container;
	private Controlador[] controladores = {null, null};
	private Joystick[] joysticks = {null, null};
	
	public Entrada(ContainerJogo container) {
		this.container = container;			
		this.teclado = new Teclado(container);		
	}
		
	public void criarJoystick1() {
		joysticks[0] = criarJoystick(0);
		controladores[0] = new Controlador(joysticks[0]);
	}
	
	public void criarJoystick2() {
		joysticks[1] = criarJoystick(1);
		controladores[1] = new Controlador(joysticks[1]);
	}
			
	public void removeJoystick1() {
		joysticks[0] = null;
	}
	
	public void removeJoystick2() {
		joysticks[1] = null;
	}
	
	public void addTecladoListener(KeyListener listener) {
		teclado.addKeyListenerTeclado(listener);
	}
	
	public void addControlador1Listener(JoystickListener listener) {		
		if(controladores[0] != null)
			controladores[0].addJoystickListenerControlador(listener);
	}
	
	public void addControlador2Listener(JoystickListener listener) {
		if(controladores[1] != null)
			controladores[1].addJoystickListenerControlador(listener);
	}
	
	public void removeTecladoListener(KeyListener listener) {
		teclado.removeKeyListenerTeclado(listener);
	}
	
	public void removeControlador1Listener(JoystickListener listener) {
		if(controladores[0] != null)
			controladores[0].removeJoystickListenerControlador(listener);
	}
	
	public void removeControlador2Listener(JoystickListener listener) {
		if(controladores[1] != null)
			controladores[1].removeJoystickListenerControlador(listener);
	}
				
	public void addEntradaListener() {
		container.getContainer().addKeyListener(teclado);		
		for(int i = 0; i < joysticks.length; i++)
			if(joysticks[i] != null && controladores[i] != null)
				joysticks[i].addJoystickListener(controladores[i]);
	}	
	
	public void removeEntradaListener() {
		container.getContainer().removeKeyListener(teclado);
		for(int i = 0; i < joysticks.length; i++)
			if(joysticks[i] != null && controladores[i] != null)
				joysticks[i].removeJoystickListener(controladores[i]);
	}

	
	public static Joystick criarJoystick(int indice) {
		Joystick joystick = null;
		try {
			joystick = Joystick.createInstance(indice);
		} catch(UnsatisfiedLinkError e) {
			JOptionPane.showMessageDialog(null, 
					"Joystick no encontrado !!!", 
					"Joystick", JOptionPane.INFORMATION_MESSAGE);
		} catch(RuntimeException e) {
			JOptionPane.showMessageDialog(null, 
					"Joystick no encontrado !!!", 
					"Joystick", JOptionPane.INFORMATION_MESSAGE);
		} catch(IOException e) {		
			e.printStackTrace();
			System.exit(1);
		}
		return joystick;
	}

	
}
 