package italo.comecome.gui.desenho;

import italo.comecome.gui.ContainerJogo;
import italo.comecome.gui.PainelPersonalizado;
import italo.comecome.gui.figura.Imagem;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class DesenhoTabuleiro implements Desenho{
	
	private static final long serialVersionUID = 1L;
	
	private final static int LARGURA_BOLINHA = 7;
	private final static int ALTURA_BOLINHA = 7;
	private final static int LARGURA_BOLINHA_GRANDE = 10;
	private final static int ALTURA_BOLINHA_GRANDE = 10;
	
	private final static Color COR_BOLINHA = Color.WHITE;
	private final static Color COR_BOLINHA_GRANDE = Color.GREEN;

	private Map<Integer, Imagem> imagensMonstrinho = null;
	private Map<Integer, Imagem> imagensComeCome = null; 
	private Imagem mensagemImg = null;
	private boolean[][] bolinhas = null;
	private int[][] bolinhasGrandes = null;
	
	private PainelPersonalizado painelPersonalizado;
	private int x1;
	private int y1;
	private int largura;
	private int altura;
	private int larguraCelula;
	private int alturaCelula;
	private String caminhoImagem;
	private ContainerJogo container;
		
	public DesenhoTabuleiro(ContainerJogo container, int x1, int y1, int largura, int altura, 
			int larguraCelula, int alturaCelula) {
		this.container = container;
		this.painelPersonalizado = container.getPainelPersonalizado();
		this.x1 = x1;
		this.y1 = y1;
		this.largura = largura;
		this.altura = altura;
		this.larguraCelula = larguraCelula;
		this.alturaCelula = alturaCelula;
		this.novaListaImagensComeCome();
		this.novaListaImagensMonstrinho();
	}
	
	public void novaListaImagensMonstrinho() {
		imagensMonstrinho = new HashMap<Integer, Imagem>();
	}
	
	public void novaListaImagensComeCome() {
		imagensComeCome = new HashMap<Integer, Imagem>();
	}
		
	public synchronized void desenharFiguras(boolean[][] bolinhas, int[][] bolinhasGrandes) {		
		setBolinhas(bolinhas);
		setBolinhasGrandes(bolinhasGrandes);
		painelPersonalizado.desenhar(this);
	}
	
	public void desenharMensagem(String caminho, int largura, int altura) { 	
		mensagemImg = container.getFigura().novaImagem(caminho);			
		mensagemImg.setDim(largura, altura);
		painelPersonalizado.desenhar(this);
	}
			
	public void apagarMensagem() {
		mensagemImg = null;
	}

	private void desenharTabuleiro(Graphics g) {
		Imagem imagem = container.getFigura().novaImagem(caminhoImagem);
		imagem.setDim(largura, altura);
		imagem.setPosicao(x1, y1);
		imagem.desenhar(g, painelPersonalizado);
	}
	
	private void desenharBolinhas(Graphics g) {
		if(bolinhas == null || bolinhasGrandes == null)
			return;
		Graphics2D g2D = (Graphics2D)g;
		g2D.setColor(COR_BOLINHA);
		for(int i = 0; i < bolinhas.length; i++)
			for(int j = 0; j < bolinhas[0].length; j++)
				if(bolinhas[i][j])
					g2D.fillArc(x1+((j+1)*larguraCelula/2), y1+((i+1)*alturaCelula/2), LARGURA_BOLINHA, ALTURA_BOLINHA, 0, 360);
		g2D.setColor(COR_BOLINHA_GRANDE);
		for(int i = 0; i < bolinhasGrandes.length; i++) {
			int xB = bolinhasGrandes[i][0];
			int yB = bolinhasGrandes[i][1];
			if(xB >= 0 && yB >= 0)
				g2D.fillArc(x1+((yB+1)*larguraCelula/2), y1+((xB+1)*alturaCelula/2), LARGURA_BOLINHA_GRANDE, ALTURA_BOLINHA_GRANDE, 0, 360);
		}	
	}
	
	private void desenharFiguras(Graphics g) {
		desenharComeComes(g);
		desenharMonstrinhos(g);
	}
	
	private void desenharComeComes(Graphics g) {
		if(imagensComeCome != null) {
			Collection<Imagem> imagens = imagensComeCome.values();
			Iterator<Imagem> i = imagens.iterator();
			while(i.hasNext()) {
				Imagem p = i.next();
				p.desenhar(g, painelPersonalizado);
			}
		}
	}
	
	private void desenharMonstrinhos(Graphics g) {
		if(imagensMonstrinho != null) {
			Collection<Imagem> imagens = imagensMonstrinho.values();			
			Iterator<Imagem> i = imagens.iterator();
			while(i.hasNext()) {
				Imagem p = i.next();
				p.desenhar(g, painelPersonalizado);			
			}
		}
	}
	
	private void desenharMensagem(Graphics g) {
		if(mensagemImg != null) {			
			ajustarDimensaoMensagem();
			mensagemImg.desenhar(g, painelPersonalizado);
		}
	}
	
	private void ajustarDimensaoMensagem() {
		int larguraImagem = mensagemImg.getLargura();
		int alturaImagem = mensagemImg.getAltura();
		int x = x1+(largura-larguraImagem)/2; 
		int y = y1+(altura-alturaImagem)/2;
		mensagemImg.setPosicao(x, y);
		mensagemImg.setDim(larguraImagem, alturaImagem);
	}
	
	public void desenhar(Graphics g) {			
		this.desenharTabuleiro(g);
		this.desenharBolinhas(g);
		this.desenharFiguras(g);
		this.desenharMensagem(g);
	}

	public boolean[][] getBolinhas() {
		return bolinhas;
	}

	public void setBolinhas(boolean[][] bolinhas) {
		this.bolinhas = bolinhas;
	}

	public int[][] getBolinhasGrandes() {
		return bolinhasGrandes;
	}

	public void setBolinhasGrandes(int[][] bolinhasGrandes) {
		this.bolinhasGrandes = bolinhasGrandes;
	}

	public int getX1() {
		return x1;
	}

	public void setX1(int x1) {
		this.x1 = x1;
	}

	public int getY1() {
		return y1;
	}

	public void setY1(int y1) {
		this.y1 = y1;
	}

	public int getLargura() {
		return largura;
	}

	public void setLargura(int largura) {
		this.largura = largura;
	}

	public int getAltura() {
		return altura;
	}

	public void setAltura(int altura) {
		this.altura = altura;
	}

	public int getLarguraCelula() {
		return larguraCelula;
	}

	public void setLarguraCelula(int larguraCelula) {
		this.larguraCelula = larguraCelula;
	}

	public int getAlturaCelula() {
		return alturaCelula;
	}

	public void setAlturaCelula(int alturaCelula) {
		this.alturaCelula = alturaCelula;
	}

	public String getCaminhoImagem() {
		return caminhoImagem;
	}

	public void setCaminhoImagem(String caminhoImagem) {
		this.caminhoImagem = caminhoImagem;
	}

	public Map<Integer, Imagem> getImagensMonstrinho() {
		return imagensMonstrinho;
	}

	public void setImagensMonstrinho(Map<Integer, Imagem> imagensMonstrinho) {
		this.imagensMonstrinho = imagensMonstrinho;
	}

	public Map<Integer, Imagem> getImagensComeCome() {
		return imagensComeCome;
	}

	public void setImagensComeCome(Map<Integer, Imagem> imagensComeCome) {
		this.imagensComeCome = imagensComeCome;
	}
	
}
