package italo.comecome.logica;

import java.util.ArrayList;
import java.util.List;

public class Jogo implements JogoIT{
	
	public final static int DIFICULDADE_FACIL = 1;
	public final static int DIFICULDADE_NORMAL = 2;
	public final static int DIFICULDADE_DIFICIL = 3;
	public final static int DIFICULDADE_SUPER_DIFICIL = 4;
	
	private Fase[] fases = null;	
	private int indiceFaseAtual = 0;	
	private int nVidas = 0;
	private int nVidasAtual = 0;
	private int dificuldade = DIFICULDADE_NORMAL;
		
	private List<JogoListener> listeners = new ArrayList<JogoListener>();
	
	public Jogo() {}

	public void inicializar() {
		this.indiceFaseAtual = 0;
		this.nVidasAtual = nVidas;
	}
			
	public void executar() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).executar(this);
	}
	
	public void iniciar() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).iniciar(this);
	}
		
	public void startar() {
		inicializar();
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).startar(this);
	}
	
	public void sair() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).sair(this);
	}
	
	public void finalizar() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).finalizar(this);
	}
	
	public void pararExecucao() {
		for(int i = 0; i < listeners.size(); i++){					
			listeners.get(i).pararExecucao(this);
		}
	}
	
	public void continuarExecucao() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).continuarExecucao(this);
	}
	
	public void zerou() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).zerou(this);
	}
	
	public void gameover() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).gameover(this);		
	}
	
	public void perdeu() {		
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).perdeu(this);		
	}

	public void passou() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).passou(this);
	}
			
	public boolean isFimJogo() {
		return (nVidasAtual <= 0 || indiceFaseAtual >= fases.length);
	}
	
	public void proximaFase() {
		indiceFaseAtual++;
	}
			
	public Fase getFaseAtual() {
		if(indiceFaseAtual < fases.length)
			return fases[indiceFaseAtual];
		return fases[fases.length-1];
	}

	public Fase[] getFases() {
		return fases;
	}

	public void setFases(Fase[] fases) {
		this.fases = fases;
	}
	
	public int getDificuldade() {
		return dificuldade;
	}

	public void setDificuldade(int dificuldade) {
		this.dificuldade = dificuldade;
	}

	public int getIndiceFaseAtual() {
		return indiceFaseAtual;
	}

	public void setIndiceFaseAtual(int indiceFaseAtual) {
		this.indiceFaseAtual = indiceFaseAtual;
	}
	
	public void addJogoListener(JogoListener listener) {
		listeners.add(listener);
	}
	
	public void removeJogoListener(JogoListener listener) {
		listeners.remove(listener);
	}

	public int getNVidas() {
		return nVidas;
	}

	public void setNVidas(int nVidas) {
		this.nVidas = nVidas;
		this.nVidasAtual = nVidas;
	}
	
	public int getNVidasAtual() {
		return nVidasAtual;
	}

	public void setNVidasAtual(int nVidas) {
		nVidasAtual = nVidas;
	}
	
}
