package italo.comecome.logica;





public class Tabuleiro {
		
	public final static int FATOR_ERRO = -1;
	
	public final static int FATOR_HORIZONTAL = 4;
	public final static int FATOR_VERTICAL = 3;
	
	private int x1 = 0;
	private int y1 = 0;
	private int larguraCompleta;
	private int alturaCompleta;
	private int larguraTabuleiro = 0;
	private int alturaTabuleiro = 0;
	private int alturaTabuleiroInicial = 0;
	private int larguraCelula = 0;
	private int alturaCelula = 0;
	private int larguraMiniCelula = 0;
	private int alturaMiniCelula = 0;	
	private int nCelulasHorizontal = 12;
	private int nCelulasVertical = 9;
	private int fatorMiniCelula = FATOR_ERRO;
	private int[] fatoresMiniCelula;
	
	public Tabuleiro(int larguraCompleta, int alturaCompleta, 
			int fatorDimTabuleiro,
			int nCelulasHorizontal, int nCelulasVertical, int[] fatoresMiniCelula) 
				throws TabuleiroException {		
		this.larguraCompleta = larguraCompleta;
		this.alturaCompleta = alturaCompleta;
		this.alturaTabuleiroInicial = this.alturaCompleta-fatorDimTabuleiro*FATOR_VERTICAL;		
		this.nCelulasHorizontal = nCelulasHorizontal;
		this.nCelulasVertical = nCelulasVertical;
		this.fatoresMiniCelula = fatoresMiniCelula;		
		this.ajustarResolucao();
	}
	
	public void ajustarDimensao() {
		alturaTabuleiro = alturaTabuleiroInicial/FATOR_VERTICAL*FATOR_VERTICAL;
		larguraTabuleiro = alturaTabuleiro*4/3;		
	}
						
	private void ajustarResolucao() throws TabuleiroException {		
		boolean ajustou = false;
		for(int i = 0; !ajustou && i < fatoresMiniCelula.length; i++) {
			ajustarDimensao();
			int fator = fatoresMiniCelula[i];			
			while(!ajustou && larguraTabuleiro > 0 && alturaTabuleiro > 0) {
				if(ajusteOk())
					ajustou = ajustarValores(fator);			
				if(!ajustou) {
					larguraTabuleiro -= FATOR_HORIZONTAL;
					alturaTabuleiro -= FATOR_VERTICAL;
				}			
			}
		}
		if(!ajustou)
			throw new TabuleiroException(this);
	}
			
	private boolean ajustarValores(int fator) {
		this.x1 = (int) ((larguraCompleta-larguraTabuleiro)/2);
		this.y1 = (int) ((alturaCompleta-alturaTabuleiro)/2);		
		this.larguraCelula = larguraTabuleiro/nCelulasHorizontal;
		this.alturaCelula = alturaTabuleiro/nCelulasVertical;		
		if(ajusteFatorMiniCelulaOk(fator)) {
			this.fatorMiniCelula = fator;
			this.larguraMiniCelula = larguraCelula/fator;
			this.alturaMiniCelula = alturaCelula/fator;
			return true;
		}
		return false;
	}
		
	private boolean ajusteOk() {
		return (larguraTabuleiro%nCelulasHorizontal == 0 && alturaTabuleiro%nCelulasVertical == 0);
	}
	
	private boolean ajusteFatorMiniCelulaOk(int fator) {
		return (larguraCelula%fator == 0 && alturaCelula%fator == 0);
	}


	public int getX1() {
		return x1;
	}


	public void setX1(int x1) {
		this.x1 = x1;
	}


	public int getY1() {
		return y1;
	}


	public void setY1(int y1) {
		this.y1 = y1;
	}


	public int getLarguraCompleta() {
		return larguraCompleta;
	}


	public void setLarguraCompleta(int larguraCompleta) {
		this.larguraCompleta = larguraCompleta;
	}


	public int getAlturaCompleta() {
		return alturaCompleta;
	}


	public void setAlturaCompleta(int alturaCompleta) {
		this.alturaCompleta = alturaCompleta;
	}


	public int getLarguraTabuleiro() {
		return larguraTabuleiro;
	}


	public void setLarguraTabuleiro(int larguraTabuleiro) {
		this.larguraTabuleiro = larguraTabuleiro;
	}


	public int getAlturaTabuleiro() {
		return alturaTabuleiro;
	}


	public void setAlturaTabuleiro(int alturaTabuleiro) {
		this.alturaTabuleiro = alturaTabuleiro;
	}


	public int getLarguraCelula() {
		return larguraCelula;
	}


	public void setLarguraCelula(int larguraCelula) {
		this.larguraCelula = larguraCelula;
	}


	public int getAlturaCelula() {
		return alturaCelula;
	}


	public void setAlturaCelula(int alturaCelula) {
		this.alturaCelula = alturaCelula;
	}


	public int getLarguraMiniCelula() {
		return larguraMiniCelula;
	}


	public void setLarguraMiniCelula(int larguraMiniCelula) {
		this.larguraMiniCelula = larguraMiniCelula;
	}


	public int getAlturaMiniCelula() {
		return alturaMiniCelula;
	}


	public void setAlturaMiniCelula(int alturaMiniCelula) {
		this.alturaMiniCelula = alturaMiniCelula;
	}

	public int getNCelulasHorizontal() {
		return nCelulasHorizontal;
	}

	public void setNCelulasHorizontal(int celulasHorizontal) {
		nCelulasHorizontal = celulasHorizontal;
	}

	public int getNCelulasVertical() {
		return nCelulasVertical;
	}

	public void setNCelulasVertical(int celulasVertical) {
		nCelulasVertical = celulasVertical;
	}

	public int getFatorMiniCelula() {
		return fatorMiniCelula;
	}

	public void setFatorMiniCelula(int fatorMiniCelula) {
		this.fatorMiniCelula = fatorMiniCelula;
	}

	public int[] getFatoresMiniCelula() {
		return fatoresMiniCelula;
	}

	public void setFatoresMiniCelula(int[] fatoresMiniCelula) {
		this.fatoresMiniCelula = fatoresMiniCelula;
	}

}
