package italo.comecome.personagem;

import italo.comecome.logica.Fase;
import italo.comecome.logica.Jogo;
import italo.comecome.logica.Tabuleiro;

public class ComeComeIMP implements ComeComeIT {

	private Jogo jogo;
	
	public ComeComeIMP(Jogo jogo) {
		this.jogo = jogo;
	}
	
	public boolean ganhou(ComeCome comecome) {
		return jogo.getFaseAtual().getNBolinhas() <= 0;		
	}
			
	public boolean monstrinhoPegou(ComeCome comecome) {		
		Fase fase = jogo.getFaseAtual();
		Tabuleiro tabuleiro = jogo.getFaseAtual().getTabuleiro();
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		int distancia = Math.min(larguraCelula, alturaCelula)*2/3;		
		Personagem[] monstrinhosProx = fase.getMonstrinhosProximos(comecome, distancia);		
		boolean pegou = false;
		if(monstrinhosProx.length > 0) {
			for(int i = 0; i < monstrinhosProx.length && !pegou; i++) {
				Monstrinho monstrinho = (Monstrinho)monstrinhosProx[i];
				if(monstrinho.getEstado() == Monstrinho.FORTE){ 
					pegou = true;
				} else if(monstrinho.getEstado() == Monstrinho.FRACO 
						|| monstrinho.getEstado() == Monstrinho.MEIO_FRACO) {
					monstrinho.setEstado(Monstrinho.MUITO_FRACO);
					fase.comeuMonstrinho();			
				}
			}
		}	
		return pegou;
	}
	
	public void mover(ComeCome comecome) {											
		comecome.mover();
	}
		
	public void comer(ComeCome comecome) {
		Fase fase = jogo.getFaseAtual();
		Tabuleiro tabuleiro = jogo.getFaseAtual().getTabuleiro();
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();	
		int x = comecome.getPosicao().getX();
		int y = comecome.getPosicao().getY();
		int xBCentro = 2*(x/larguraCelula);
		int yBCentro = 2*(y/alturaCelula);
		int xBolinha = -1;
		int yBolinha = -1;
		if(comecome.getDirecao() == Personagem.TRAZ) {
			yBolinha = yBCentro;
			 if(x % larguraCelula < larguraCelula*1/2) {
				xBolinha = xBCentro;
			} else {
				xBolinha = xBCentro+1;
			}
		} else if(comecome.getDirecao() == Personagem.FRENTE) {
			yBolinha = yBCentro;
			if(x % larguraCelula < larguraCelula*1/2) {
				xBolinha = xBCentro;
			} else {
				xBolinha = xBCentro+1;
			}
		} else if(comecome.getDirecao() == Personagem.CIMA) {
			xBolinha = xBCentro;
			if(y % alturaCelula < alturaCelula*1/2) {
				yBolinha = yBCentro;
			} else {				
				yBolinha = yBCentro+1;
			} 				
		} else if(comecome.getDirecao() == Personagem.BAIXO) {
			xBolinha = xBCentro;
			if(y % alturaCelula < alturaCelula*1/2) {
				yBolinha = yBCentro;
			} else {				
				yBolinha = yBCentro+1;
			} 			
		}	
		fase.comerBolinha(xBolinha, yBolinha);
	}
		
	public void dormir(ComeCome comecome) {
		try {
			Thread.sleep(comecome.getSleep()); 
		} catch (InterruptedException e) {
			
		}
	}

}
