package italo.comecome.personagem;

import italo.comecome.logica.Fase;
import italo.comecome.logica.Jogo;
import italo.comecome.logica.Tabuleiro;
import italo.comecome.personagem.util.MaisLonge;
import italo.comecome.personagem.util.MaisProxima;
import italo.comecome.personagem.util.PersonagemUtil;
import italo.comecome.utilitarios.Utilitarios;

public class MonstrinhoIMP implements MonstrinhoIT {

	private final static int TEMPO_MONSTRINHOS_FRACOS_INICIAL = 0;
	private final static int TEMPO_MONSTRINHOS_FRACOS_LIMITE = 100;	
	
	private int tempoMonstrinhosFracos = TEMPO_MONSTRINHOS_FRACOS_INICIAL;
	private Jogo jogo;
	
	public MonstrinhoIMP(Jogo jogo) {		
		this.jogo = jogo;
	}

	public void mover(Monstrinho monstrinho) {
		Fase fase = jogo.getFaseAtual();
		int x = monstrinho.getPosicao().getX();
		int y = monstrinho.getPosicao().getY();	
		if(fase.isPosicaoCelula(x, y)) { 
			alterarPosicaoParaMover(monstrinho);						
			paraCasa(monstrinho);	
		}
		monstrinho.mover();			
	}
	
	public void tratarEstado(Monstrinho monstrinho) {	
		switch(monstrinho.getEstado()) {
			case Monstrinho.FORTE:
				monstrinhoForte();
				break;			
			case Monstrinho.MEIO_FRACO:
			case Monstrinho.FRACO:
				monstrinhoFraco(monstrinho);	
				break;
		}				
	}
	
	public void dormir(Monstrinho monstrinho) {
		int sleep = monstrinho.getSleep();
		int estado = monstrinho.getEstado();
		switch(estado) {
			case Monstrinho.FORTE:
				Utilitarios.sleep(sleep);
				break;
			case Monstrinho.MUITO_FRACO:
				Utilitarios.sleep(sleep/5);
				break;
			default:
				Utilitarios.sleep(sleep*8/5);
		}		
	}
	
	private void alterarPosicaoParaMover(Monstrinho monstrinho) {
		int dificuldade = jogo.getDificuldade();
		Fase fase = jogo.getFaseAtual();
		ComeCome comecome = fase.getComeCome();
		double distancia = PersonagemUtil.distancia(monstrinho, comecome);
		double distanciaLimite = fase.getDistanciaLimite(dificuldade);
		int novaDir;
		if(distancia < distanciaLimite) {
			novaDir = novaDirecaoDistanciaPequena(monstrinho);
		} else {
			novaDir = novaDirecaoDistanciaGrande(monstrinho);
		}	
		monstrinho.setNovaDirecao(novaDir);
	}
	
	private int novaDirecaoDistanciaGrande(Monstrinho monstrinho) {
		Fase fase = jogo.getFaseAtual();
		Tabuleiro tabuleiro = jogo.getFaseAtual().getTabuleiro();
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		int x = monstrinho.getPosicao().getX();
		int y = monstrinho.getPosicao().getY();
		int direcao = monstrinho.getDirecao();
		int novaDir = direcao;
		boolean novaDirLivre = fase.isCelulaLivre(y/alturaCelula, x/larguraCelula, monstrinho.getNovaDirecao());		
		if(!novaDirLivre) { 			
			novaDir = PersonagemUtil.getDirecaoOposta(direcao);	
			novaDirLivre = fase.isCelulaLivre(y/alturaCelula, x/larguraCelula, novaDir);
			int cont = 5;
			while (!novaDirLivre || novaDir == PersonagemUtil.getDirecaoOposta(direcao)){ 
				novaDir = PersonagemUtil.getDirecaoAleatoria();
				novaDirLivre = fase.isCelulaLivre(y/alturaCelula, x/larguraCelula, novaDir);
				cont++;
			}
			cont = cont%5+1;				
		}
		return novaDir;
	}
	
	private int novaDirecaoDistanciaPequena(Monstrinho monstrinho) {
		Fase fase = jogo.getFaseAtual();	
		Tabuleiro tabuleiro = jogo.getFaseAtual().getTabuleiro();
		ComeCome comecome = fase.getComeCome();
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		switch(monstrinho.getEstado()) {
			case Monstrinho.FORTE:
				return PersonagemUtil.getDirecaoMelhor(monstrinho, comecome, larguraCelula, alturaCelula, new MaisProxima()); 			
			case Monstrinho.MEIO_FRACO:
			case Monstrinho.FRACO:
				return PersonagemUtil.getDirecaoMelhor(monstrinho, comecome, larguraCelula, alturaCelula, new MaisLonge());
			default:
				return novaDirecaoDistanciaGrande(monstrinho);
		}
	}
			
	private void paraCasa(Monstrinho monstrinho) {
		if(monstrinho.getEstado() == Monstrinho.MUITO_FRACO) { 
			Fase fase = jogo.getFaseAtual();
			Tabuleiro tabuleiro = jogo.getFaseAtual().getTabuleiro();
			int larguraCelula = tabuleiro.getLarguraCelula();
			int alturaCelula = tabuleiro.getAlturaCelula();
			int x = monstrinho.getPosicao().getX();
			int y = monstrinho.getPosicao().getY();
			if(fase.podeEntrarNaCasa(x/larguraCelula, y/alturaCelula)) {
				fase.entrarNaCasa(monstrinho);
				monstrinho.setEstado(Monstrinho.FORTE);
				fase.sairDaCasa(monstrinho);
			}
		}
	}
			
	private void monstrinhoForte() {
		tempoMonstrinhosFracos = TEMPO_MONSTRINHOS_FRACOS_INICIAL;
	}
	
	private void monstrinhoFraco(Monstrinho monstrinho) {
		switch(tempoMonstrinhosFracos) {
			case TEMPO_MONSTRINHOS_FRACOS_LIMITE/2:
				monstrinho.setEstado(Monstrinho.MEIO_FRACO);
				break;
			case TEMPO_MONSTRINHOS_FRACOS_LIMITE:
				monstrinho.setEstado(Monstrinho.FORTE);
				break;
		}		
		tempoMonstrinhosFracos++;
	}
	
}
