package italo.comecome.personagem.util;

import italo.comecome.personagem.Monstrinho;
import italo.comecome.personagem.Personagem;
import italo.comecome.utilitarios.Utilitarios;

public class PersonagemUtil {
	
	public static int getDirecaoAleatoria() {
		return Utilitarios.random(Personagem.DIRECAO_MINIMA, Personagem.DIRECAO_MAXIMA);
	}
	
	public static int getDirecaoOposta(int direcao) {
		switch(direcao) {
			case Personagem.FRENTE:	return Personagem.TRAZ;
			case Personagem.TRAZ:   return Personagem.FRENTE;
			case Personagem.BAIXO:  return Personagem.CIMA;
			case Personagem.CIMA:   return Personagem.BAIXO;
			default:  return -1;
		}
	}
	
	public static String getDirecao(int direcao) {
		switch(direcao) {
			case Personagem.FRENTE:	return "frente";
			case Personagem.TRAZ:   return "traz";
			case Personagem.BAIXO:  return "baixo";
			case Personagem.CIMA:   return "cima";
			default:  return "";
		}
	}
			
	public static String getEstado(int estado) {
		switch(estado) {
			case Monstrinho.FORTE:       return "forte";
			case Monstrinho.MEIO_FRACO:  return "meiofraco";	
			case Monstrinho.FRACO:       return "fraco";	
			case Monstrinho.MUITO_FRACO: return "muitofraco";	
			default: return "";
		}		
	}
	
	public static int getDirecao(String direcao) {		
		if(direcao.equals("frente")) {
			return Personagem.FRENTE;
		} else if(direcao.equals("traz")) {
			return Personagem.TRAZ;
		} else if(direcao.equals("baixo")) {
			return Personagem.BAIXO;
		} else if(direcao.equals("cima")) {
			return Personagem.CIMA;
		}
		return -1;
	}
	
	public static double distancia(Personagem p1, Personagem p2) {
		int xP1 = p1.getPosicao().getX();
		int yP1 = p1.getPosicao().getY();
		int xP2 = p2.getPosicao().getX();
		int yP2 = p2.getPosicao().getY();
		return Utilitarios.distancia(xP1, yP1, xP2, yP2);		
	}
	
	public static int getDirecaoMelhor(Personagem p1, Personagem p2, int larguraCelula, int alturaCelula, Melhor melhor) {		
		int x1 = p1.getPosicao().getX()+(larguraCelula/2);
		int y1 = p1.getPosicao().getY()+(alturaCelula/2);
		int x2 = p2.getPosicao().getX()+(larguraCelula/2);
		int y2 = p2.getPosicao().getY()+(alturaCelula/2);
		int[] direcoes = {Monstrinho.FRENTE, Monstrinho.BAIXO, Monstrinho.TRAZ, Monstrinho.CIMA};
		int[] posicoesIncX = {larguraCelula/3, 0, -alturaCelula/3, 0};
		int[] posicoesIncY = {0, alturaCelula/3, 0, -alturaCelula/3};
		int melhorDir = direcoes[0];	
		double menorDist = Utilitarios.distancia(x1+posicoesIncX[0], y1+posicoesIncY[0], x2, y2);		
		for(int i = 1; i < direcoes.length; i++) {
			if(direcoes[i] == p1.getDirecao()) 
				continue;
			double dist = Utilitarios.distancia(x1+posicoesIncX[i], y1+posicoesIncY[i], x2, y2);
			if(melhor.comparacao(dist, menorDist)) { 
				menorDist = dist;
				melhorDir = direcoes[i];
			}
		}					
		return (melhorDir);		
	}
		
}
