package italo.comecome.thread;

import italo.comecome.logica.FaseIT;
import italo.comecome.logica.JogoIT;
import italo.comecome.personagem.PersonagemIT;

public class Gerenciador {
	
	private static Gerenciador gerenciador;
	
	private ThreadPersonagem[] monstrinhosT;
	private ThreadPersonagem comecomeT;
	private ThreadJogo jogoT;
	private ThreadFase faseT;
	
	static {
		gerenciador = new Gerenciador();
	}
	
	private Gerenciador() {}
	
	public static Gerenciador getInstance() {
		return gerenciador;
	}
	
	public void iniciarNovoJogo(JogoIT jogo) {
		jogoT = new ThreadJogo(jogo);
	}
	
	public void iniciarNovaFase(FaseIT fase) {
		faseT = new ThreadFase(fase);
	}
	
	public void iniciarNovoComeCome(PersonagemIT comecome) {
		comecomeT = new ThreadPersonagem(comecome);		
	}
	
	public void iniciarNovosMonstrinhos(PersonagemIT[] monstrinhos) {
		monstrinhosT = new ThreadPersonagem[monstrinhos.length];
		for(int i = 0; i < monstrinhosT.length; i++)
			monstrinhosT[i] = new ThreadPersonagem(monstrinhos[i]);		
	}
	
	public void startJogo() {
		if(jogoT != null) {
			jogoT.start();
		}
	}
	
	public void startFase() {
		if(faseT != null)
			faseT.start();
	}
	
	public void startComeCome() {
		if(comecomeT != null)
			comecomeT.start();
	}
	
	
	public void startMonstrinho(int i) {
		if(monstrinhosT != null && i < monstrinhosT.length && monstrinhosT[i] != null)
			monstrinhosT[i].start();
	}
	
	public void startMonstrinhos() {
		if(monstrinhosT != null) {
			for(int i = 0; i < monstrinhosT.length; i++) {
				if(monstrinhosT[i] != null)
					monstrinhosT[i].start();
			}
		}
	}
		
	public void waitJogo() {
		if(jogoT != null) {
			jogoT.esperar();
		}
	}
	
	public void waitFase() {
		if(faseT != null) {
			faseT.esperar();
		}
	}
	
	public void waitComeCome() {
		if(comecomeT != null) {
			comecomeT.esperar();
		}
	}
	
	public void waitMonstrinho(int i) {
		if(monstrinhosT != null && i < monstrinhosT.length && monstrinhosT[i] != null)
			monstrinhosT[i].esperar();
	}
	
	public void waitMonstrinhos() {
		if(monstrinhosT != null ) {
			for(int i = 0; i < monstrinhosT.length; i++) {
				if(monstrinhosT[i] != null)
					monstrinhosT[i].esperar();
			}
		}
	}
	
	public void notificarJogo() {
		if(jogoT != null)			
			jogoT.notificar();		
	}
	
	public void notifyFase() {
		if(faseT != null) 
			faseT.notificar();		
	}
	
	public void notifyComeCome() {
		if(comecomeT != null)
			comecomeT.notificar();
	}
	
	public void notifyMonstrinho(int i) {
		if(monstrinhosT != null && i < monstrinhosT.length && monstrinhosT[i] != null)
			monstrinhosT[i].notificar();
	}
	
	public void notifyMonstrinhos() {
		if(monstrinhosT != null ) {
			for(int i = 0; i < monstrinhosT.length; i++) {
				if(monstrinhosT[i] != null)
					monstrinhosT[i].notificar();
			}
		}
	}

}
